/*************************************************************************
 * This file is part of Crosroads 1.38, a load balancer and fail over
 * utility for TCP. Copyright (c) Karel Kubat, distributed under GPL.
 * Visit http://crossroads.e-tunity.com for information.
 *************************************************************************/
#include "crossroads.h"

char *fmt_expand (char const *s, int *skip, int target_backend) {
    static char buf[80];
    char *ret;

    /* Expansion table:
     * %a	- availability of current back end (if any), 0 not available,
     *		  %1a = first back end etc.
     * %b	- name of current back end (if any), %1b = first back end etc
     * %e	- seconds since epoch
     * %n	- nr of clients of current back end, %1n = first back end etc.
     * %r 	- client IP
     * %s	- name of service
     * %t, %T	- timestamp of local or GMT time
     * %v	- Crossroads version
     * %w	- weight of current back end, %1w = first back end etc.
     */

    *skip = 1;
    switch (*s) {

    case 'a':
	if (backend_available (target_backend))
	    return ("1");
	return ("0");
	
    case 'b':
	if (target_backend >= 0 && target_backend < activeservice->nbackend)
	    return (activeservice->backend[target_backend].name);
	return ("noname");

    case 'e':
	snprintf (buf, sizeof(buf) - 1, "%u", (unsigned) time(0));
	return (buf);

    case 'n':
	if (target_backend >= 0 && target_backend < activeservice->nbackend) {
	    snprintf (buf, sizeof(buf) - 1, "%u",
		      (unsigned)
		        servicereport->backendstate[target_backend].nclients);
	    return (buf);
	}
	return ("0");
	
    case 'r':
	if (client_ip && *client_ip)
	    return (client_ip);
	return ("0.0.0.0");

    case 's':
	return (activeservice->name);

    case 't':
	return (ansistamp (tm_localtime));

    case 'T':
	return (ansistamp (tm_gmtime));

    case 'v':
	return (VER);

    case 'w':
	if (target_backend >= 0 && target_backend < activeservice->nbackend) {
	    snprintf (buf, sizeof(buf) - 1, "%u",
		      (unsigned)
		        activeservice->backend[target_backend].weight);
	    return (buf);
	}
	return ("0");
	
    default:
	if (isdigit (*s)) {
	    ret = fmt_expand (s + 1, skip, *s - '0' - 1);
	    *skip += 1;
	    return (ret);
	} else {
	    *buf = *(s + 1);
	    *(buf + 1) = 0;
	    return (buf);
	}
    }
}

char *str_expand_format (char const *h) {
    char *ret = 0;
    char const *cp;
    int skip;

    for (cp = h; cp && *cp; cp++) {
	if (*cp == '%') {
	    ret = xstrcat (ret, fmt_expand (cp + 1, &skip, current_backend));
	    cp += skip;
	} else {
	    ret = xstrcatch (ret, *cp);
	}
    }

    return (ret);
}
